#include "driver_calc.hh"
#include "retrial.hh"
#include "roots.hh"
#include "twopi.hh"
#include "log.hh"
#include <error.h>

Interval::Lohi Driver::calculate_by_name(
    mp_bitcnt_t *const ptr_precision,
    const std::string &name,
    const std::list<long> &parameters,
    const bool prefer_negative_branch,
    Interval::Order *const ptr_order
)
{
    Interval::Lohi result {0};
    const mp_bitcnt_t &precision {*ptr_precision};
    if (name == "quadroot") {
        if (parameters.size() != 4) error(
            EXIT_FAILURE, EINVAL,
            "%s wants four parameters",
            name.c_str()
        );
        std::list<long>::const_iterator ptr {parameters.begin()};
        const long &n {*ptr++};
        const long &s {*ptr++};
        const long &p {*ptr++};
        const long &q {*ptr++};
        result = Interval::root(
            Interval::Indicant_re_general_quadratic
            (precision, n, s, p, q, prefer_negative_branch)
        );
    }
    else if (name == "power") {
        if (parameters.size() != 4) error(
            EXIT_FAILURE, EINVAL,
            "%s wants four parameters",
            name.c_str()
        );
        std::list<long>::const_iterator ptr {parameters.begin()};
        const long &n {*ptr++};
        const long &m {*ptr++};
        const long &p {*ptr++};
        const long &q {*ptr++};
        result = Interval::root(
            Interval::Indicant_re_power
            (precision, n, m, p, q, prefer_negative_branch)
        );
    }
    else if (name == "twopi") {
        if (!parameters.empty()) error(
            EXIT_FAILURE, EINVAL,
            "%s wants no parameters",
            name.c_str()
        );
        result = Interval::retry
          (Interval::calc_twopi, ptr_precision, ptr_order);
    }
    else if (name == "sqrttwopi") {
        if (!parameters.empty()) error(
            EXIT_FAILURE, EINVAL,
            "%s wants no parameters",
            name.c_str()
        );
        result = Interval::retry
          (Interval::calc_sqrttwopi, ptr_precision, ptr_order);
    }
    else if (name == "log") {
        if (parameters.size() != 2) error(
            EXIT_FAILURE, EINVAL,
            "%s wants two parameters",
            name.c_str()
        );
        std::list<long>::const_iterator ptr {parameters.begin()};
        const long &p {*ptr++};
        const long &q {*ptr++};
        result = Interval::retry
          (Interval::calc_log, ptr_precision, p, q, ptr_order);
    }
    else if (name == "logtwopi") {
        if (!parameters.empty()) error(
            EXIT_FAILURE, EINVAL,
            "%s wants no parameters",
            name.c_str()
        );
        result = Interval::retry
          (Interval::calc_logtwopi, ptr_precision, ptr_order);
    }
    else error(EXIT_FAILURE, EINVAL, "unknown name");
    return result;
}

