#include "../../src/divrem.hh"
#include <cassert>

int main()
{
    { const Interval::Quotrem<long, long> a {Interval::divrem(+14, +3)}; assert(a.quot == +4 && a.rem == +2); }
    { const Interval::Quotrem<long, long> a {Interval::divrem(+14, -3)}; assert(a.quot == -5 && a.rem == -1); }
    { const Interval::Quotrem<long, long> a {Interval::divrem(-14, +3)}; assert(a.quot == -5 && a.rem == +1); }
    { const Interval::Quotrem<long, long> a {Interval::divrem(-14, -3)}; assert(a.quot == +4 && a.rem == -2); }
    { const Interval::Quotrem<long, long> a {Interval::divrem( +2, +3)}; assert(a.quot ==  0 && a.rem == +2); }
    { const Interval::Quotrem<long, long> a {Interval::divrem( +2, -3)}; assert(a.quot == -1 && a.rem == -1); }
    { const Interval::Quotrem<long, long> a {Interval::divrem( -2, +3)}; assert(a.quot == -1 && a.rem == +1); }
    { const Interval::Quotrem<long, long> a {Interval::divrem( -2, -3)}; assert(a.quot ==  0 && a.rem == -2); }
    { const Interval::Quotrem<long, long> a {Interval::divrem(+18, +3)}; assert(a.quot == +6 && a.rem ==  0); }
    { const Interval::Quotrem<long, long> a {Interval::divrem(+18, -3)}; assert(a.quot == -6 && a.rem ==  0); }
    { const Interval::Quotrem<long, long> a {Interval::divrem(-18, +3)}; assert(a.quot == -6 && a.rem ==  0); }
    { const Interval::Quotrem<long, long> a {Interval::divrem(-18, -3)}; assert(a.quot == +6 && a.rem ==  0); }
    { const Interval::Quotrem<long, long> a {Interval::divrem(  0, +3)}; assert(a.quot ==  0 && a.rem ==  0); }
    { const Interval::Quotrem<long, long> a {Interval::divrem(  0, -3)}; assert(a.quot ==  0 && a.rem ==  0); }
    { const Interval::Quotrem<long, long> a {Interval::divrem(  0, +3)}; assert(a.quot ==  0 && a.rem ==  0); }
    { const Interval::Quotrem<long, long> a {Interval::divrem(  0, -3)}; assert(a.quot ==  0 && a.rem ==  0); }
    {       Interval::Quotrem<long, long> a {Interval::divrem(+14, -3)}; assert(a.quot == -5 && a.rem == -1); }
    {
        int n_thrown {0};
        try { [[maybe_unused]] const Interval::Quotrem<long, long> a {Interval::divrem(+14, 0)}; }
        catch (const Interval::Division_by_zero) { ++n_thrown; }
        try { [[maybe_unused]] const Interval::Quotrem<long, long> a {Interval::divrem(  0, 0)}; }
        catch (const Interval::Division_by_zero) { ++n_thrown; }
        assert(n_thrown == 2);
    }
    { const Interval::Quotrem<mpz_class, long> a {Interval::divrem(mpz_class {+14}, +3)}; assert(a.quot == +4 && a.rem == +2); }
    { const Interval::Quotrem<mpz_class, long> a {Interval::divrem(mpz_class {+14}, -3)}; assert(a.quot == -5 && a.rem == -1); }
    { const Interval::Quotrem<mpz_class, long> a {Interval::divrem(mpz_class {-14}, +3)}; assert(a.quot == -5 && a.rem == +1); }
    { const Interval::Quotrem<mpz_class, long> a {Interval::divrem(mpz_class {-14}, -3)}; assert(a.quot == +4 && a.rem == -2); }
    { const Interval::Quotrem<mpz_class, long> a {Interval::divrem(mpz_class { +2}, +3)}; assert(a.quot ==  0 && a.rem == +2); }
    { const Interval::Quotrem<mpz_class, long> a {Interval::divrem(mpz_class { +2}, -3)}; assert(a.quot == -1 && a.rem == -1); }
    { const Interval::Quotrem<mpz_class, long> a {Interval::divrem(mpz_class { -2}, +3)}; assert(a.quot == -1 && a.rem == +1); }
    { const Interval::Quotrem<mpz_class, long> a {Interval::divrem(mpz_class { -2}, -3)}; assert(a.quot ==  0 && a.rem == -2); }
    { const Interval::Quotrem<mpz_class, long> a {Interval::divrem(mpz_class {+18}, +3)}; assert(a.quot == +6 && a.rem ==  0); }
    { const Interval::Quotrem<mpz_class, long> a {Interval::divrem(mpz_class {+18}, -3)}; assert(a.quot == -6 && a.rem ==  0); }
    { const Interval::Quotrem<mpz_class, long> a {Interval::divrem(mpz_class {-18}, +3)}; assert(a.quot == -6 && a.rem ==  0); }
    { const Interval::Quotrem<mpz_class, long> a {Interval::divrem(mpz_class {-18}, -3)}; assert(a.quot == +6 && a.rem ==  0); }
    { const Interval::Quotrem<mpz_class, long> a {Interval::divrem(mpz_class {  0}, +3)}; assert(a.quot ==  0 && a.rem ==  0); }
    { const Interval::Quotrem<mpz_class, long> a {Interval::divrem(mpz_class {  0}, -3)}; assert(a.quot ==  0 && a.rem ==  0); }
    { const Interval::Quotrem<mpz_class, long> a {Interval::divrem(mpz_class {  0}, +3)}; assert(a.quot ==  0 && a.rem ==  0); }
    { const Interval::Quotrem<mpz_class, long> a {Interval::divrem(mpz_class {  0}, -3)}; assert(a.quot ==  0 && a.rem ==  0); }
    {       Interval::Quotrem<mpz_class, long> a {Interval::divrem(mpz_class {+14}, -3)}; assert(a.quot == -5 && a.rem == -1); }
    {
        int n_thrown {0};
        try { [[maybe_unused]] const Interval::Quotrem<mpz_class, long> a {Interval::divrem(mpz_class {+14}, 0)}; }
        catch (const Interval::Division_by_zero) { ++n_thrown; }
        try { [[maybe_unused]] const Interval::Quotrem<mpz_class, long> a {Interval::divrem(mpz_class {  0}, 0)}; }
        catch (const Interval::Division_by_zero) { ++n_thrown; }
        assert(n_thrown == 2);
    }
    { const Interval::Quotrem<mpz_class, mpz_class> a {Interval::divrem(mpz_class {+14}, mpz_class {+3})}; assert(a.quot == +4 && a.rem == +2); }
    { const Interval::Quotrem<mpz_class, mpz_class> a {Interval::divrem(mpz_class {+14}, mpz_class {-3})}; assert(a.quot == -5 && a.rem == -1); }
    { const Interval::Quotrem<mpz_class, mpz_class> a {Interval::divrem(mpz_class {-14}, mpz_class {+3})}; assert(a.quot == -5 && a.rem == +1); }
    { const Interval::Quotrem<mpz_class, mpz_class> a {Interval::divrem(mpz_class {-14}, mpz_class {-3})}; assert(a.quot == +4 && a.rem == -2); }
    { const Interval::Quotrem<mpz_class, mpz_class> a {Interval::divrem(mpz_class { +2}, mpz_class {+3})}; assert(a.quot ==  0 && a.rem == +2); }
    { const Interval::Quotrem<mpz_class, mpz_class> a {Interval::divrem(mpz_class { +2}, mpz_class {-3})}; assert(a.quot == -1 && a.rem == -1); }
    { const Interval::Quotrem<mpz_class, mpz_class> a {Interval::divrem(mpz_class { -2}, mpz_class {+3})}; assert(a.quot == -1 && a.rem == +1); }
    { const Interval::Quotrem<mpz_class, mpz_class> a {Interval::divrem(mpz_class { -2}, mpz_class {-3})}; assert(a.quot ==  0 && a.rem == -2); }
    { const Interval::Quotrem<mpz_class, mpz_class> a {Interval::divrem(mpz_class {+18}, mpz_class {+3})}; assert(a.quot == +6 && a.rem ==  0); }
    { const Interval::Quotrem<mpz_class, mpz_class> a {Interval::divrem(mpz_class {+18}, mpz_class {-3})}; assert(a.quot == -6 && a.rem ==  0); }
    { const Interval::Quotrem<mpz_class, mpz_class> a {Interval::divrem(mpz_class {-18}, mpz_class {+3})}; assert(a.quot == -6 && a.rem ==  0); }
    { const Interval::Quotrem<mpz_class, mpz_class> a {Interval::divrem(mpz_class {-18}, mpz_class {-3})}; assert(a.quot == +6 && a.rem ==  0); }
    { const Interval::Quotrem<mpz_class, mpz_class> a {Interval::divrem(mpz_class {  0}, mpz_class {+3})}; assert(a.quot ==  0 && a.rem ==  0); }
    { const Interval::Quotrem<mpz_class, mpz_class> a {Interval::divrem(mpz_class {  0}, mpz_class {-3})}; assert(a.quot ==  0 && a.rem ==  0); }
    { const Interval::Quotrem<mpz_class, mpz_class> a {Interval::divrem(mpz_class {  0}, mpz_class {+3})}; assert(a.quot ==  0 && a.rem ==  0); }
    { const Interval::Quotrem<mpz_class, mpz_class> a {Interval::divrem(mpz_class {  0}, mpz_class {-3})}; assert(a.quot ==  0 && a.rem ==  0); }
    {       Interval::Quotrem<mpz_class, mpz_class> a {Interval::divrem(mpz_class {+14}, mpz_class {-3})}; assert(a.quot == -5 && a.rem == -1); }
    {
        int n_thrown {0};
        try { [[maybe_unused]] const Interval::Quotrem<mpz_class, mpz_class> a {Interval::divrem(mpz_class {+14}, mpz_class{0})}; }
        catch (const Interval::Division_by_zero) { ++n_thrown; }
        try { [[maybe_unused]] const Interval::Quotrem<mpz_class, mpz_class> a {Interval::divrem(mpz_class {  0}, mpz_class{0})}; }
        catch (const Interval::Division_by_zero) { ++n_thrown; }
        assert(n_thrown == 2);
    }
    return 0;
}

