#include "../../src/lohi.hh"
#include <cassert>

int main()
{

    // Interval::Prelohi::lo(), hi(), ne(), fa(), and const
    {
        const Interval::Prelohi a {28, 33};
        assert(a.lo() == a.ne() && a.hi() == a.fa());
        assert(a.lo() == 28 && a.hi() == 33);
        Interval::Prelohi b {53, 38};
        ++b.lo();
        assert(b.lo() == b.ne() && b.hi() == b.fa());
        assert(b.lo() == 54 && b.hi() == 38);
    }

    // Interval::Prelohi::Prelohi()
    { const Interval::Prelohi a; assert(a.lo() == 0 && a.hi() == 0); }

    // Interval::Prelohi::Prelohi(const mpz_class &)
    { const Interval::Prelohi a {31}; assert(a.lo() == 31 && a.hi() == 31); }

    // Interval::Prelohi::Prelohi(const mpz_class &, const mpz_class &)
    {
        Interval::Prelohi a {28, 33}; assert(a.lo() == 28 && a.hi() == 33);
        const Interval::Prelohi b {33, 28}; assert(b.lo() == 33 && b.hi() == 28);
    }

    // Interval::Prelohi::swap()
    { Interval::Prelohi a {33, 28}; ++a.swap().lo(); assert(a.lo() == 29 && a.hi() == 33); }

    // Interval::Lohi::lo() const, hi() const
    {
        const Interval::Lohi a {28, 33}; assert(a.lo() == 28 && a.hi() == 33);
        Interval::Lohi b {38, 43}; assert(b.lo() == 38 && b.hi() == 43);
        const mpz_class &v = b.hi(); assert(v == 43 && &v == &b.hi());
    }

    // Interval::Lohi_bad_state
    {
        bool has_thrown {false};
        try { const Interval::Lohi a {27, 31}; }
        catch (const Interval::Lohi_bad_state) { has_thrown = true; }
        assert (!has_thrown);
        try { const Interval::Lohi a {31, 31}; }
        catch (const Interval::Lohi_bad_state) { has_thrown = true; }
        assert (!has_thrown);
        try { const Interval::Lohi a {31, 27}; }
        catch (const Interval::Lohi_bad_state) { has_thrown = true; }
        assert (has_thrown);
    }

    // Interval::Lohi::Lohi()
    { const Interval::Lohi a; assert(a.lo() == 0 && a.hi() == 0); }

    // Interval::Lohi::Lohi(const mpz_class &)
    { const Interval::Lohi a {31}; assert(a.lo() == 31 && a.hi() == 31); }

    // Interval::Lohi::Lohi(const mpz_class &, const mpz_class &)
    {
        const Interval::Lohi a {28, 33}; assert(a.lo() == 28 && a.hi() == 33);
        Interval::Lohi b {38, 53}; assert(b.lo() == 38 && b.hi() == 53);
    }

    // Interval::Lohi::Lohi(const Prelohi &)
    {
        const Interval::Prelohi p {28, 33}; Interval::Lohi a {p}; assert(a.lo() == 28 && a.hi() == 33);
        Interval::Prelohi q {53, 38}; q.swap(); const Interval::Lohi b {q}; assert(b.lo() == 38 && b.hi() == 53);
        bool has_thrown = false;
        try { Interval::Prelohi r {10, 50}; r.swap(); const Interval::Lohi c {r}; }
        catch (const Interval::Lohi_bad_state) { has_thrown = true; }
        assert(has_thrown);
    }

    // Interval::Lohi_bad_state
    // (This test is probably superfluous, but that's okay.)
    {
        const Interval::Prelohi p {33, 28};
        bool has_thrown {false};
        try { const Interval::Lohi a {p}; }
        catch (const Interval::Lohi_bad_state) { has_thrown = true; }
        assert(has_thrown);
    }

    // Objects used during later tests.
    [[maybe_unused]] const Interval::Lohi n {27, 31}; assert(n.lo() == 27 && n.hi() == 31);
    [[maybe_unused]] const Interval::Lohi m {6, 7}; assert(m.lo() == 6 && m.hi() == 7);
    [[maybe_unused]] const mpz_class j {10};
    [[maybe_unused]] const long k {4};
    [[maybe_unused]] const mp_bitcnt_t bc {3};

    // Interval::Lohi::operator=(const mpz_class &)
    {
        Interval::Lohi a {n}; assert(a.lo() == 27 && a.hi() == 31);
        a = j; assert(a.lo() == 10 && a.hi() == 10);
        a = n; assert(a.lo() == 27 && a.hi() == 31);
        a = a.lo(); assert(a.lo() == 27 && a.hi() == 27);
        a = n; assert(a.lo() == 27 && a.hi() == 31);
        a = a.hi(); assert(a.lo() == 31 && a.hi() == 31);
    }

    // Interval::Lohi::operator+=(const Lohi &)
    { Interval::Lohi a {n}; a += m; assert(a.lo() == 33 && a.hi() == 38); }

    // Interval::Lohi::operator+=(const mpz_class &)
    {
        Interval::Lohi a {n}; a += j; assert(a.lo() == 37 && a.hi() == 41);
        a += a.lo(); assert(a.lo() == 74 && a.hi() == 78);
        a += a.hi(); assert(a.lo() == 152 && a.hi() == 156);
    }

    // Interval::Lohi::operator-=(const Lohi &)
    {
        Interval::Lohi a {n}; a -= m; assert(a.lo() == 20 && a.hi() == 25);
        Interval::Lohi b {n}; b -= b; assert(b.lo() == -4 && b.hi() == 4);
        // Apparently, to write b -= b is in most circumstances a bad idea!
        // However, the point here has been to verify that the bad idea
        // returns the correct bad result.
    }

    // Interval::Lohi::operator-=(const mpz_class &)
    {
        Interval::Lohi a {n}; a -= j; assert(a.lo() == 17 && a.hi() == 21);
        a -= a.lo(); assert(a.lo() == 0 && a.hi() == 4);
        a -= a.hi(); assert(a.lo() == -4 && a.hi() == 0);
        a = n; a -= j; assert(a.lo() == 17 && a.hi() == 21);
        a -= a.hi(); assert(a.lo() == -4 && a.hi() == 0);
    }

    // Interval::Lohi::operator*=(const Lohi &)
    {
        using Interval::Lohi;
        Lohi a, b;
        a = Lohi { 100,  200}; b = Lohi { 10,  20}; a *= b; assert(a.lo() ==  1000 && a.hi() ==  4000);
        a = Lohi { 100,  200}; b = Lohi {-10,  13}; a *= b; assert(a.lo() == -2000 && a.hi() ==  2600);
        a = Lohi { 100,  200}; b = Lohi {-20, -10}; a *= b; assert(a.lo() == -4000 && a.hi() == -1000);
        a = Lohi {-100,  103}; b = Lohi { 10,  20}; a *= b; assert(a.lo() == -2000 && a.hi() ==  2060);
        a = Lohi {-100,  103}; b = Lohi {-10,  13}; a *= b; assert(a.lo() == -1300 && a.hi() ==  1339);
        a = Lohi {-100,  103}; b = Lohi {-13,  10}; a *= b; assert(a.lo() == -1339 && a.hi() ==  1300);
        a = Lohi {-103,  100}; b = Lohi {-10,  13}; a *= b; assert(a.lo() == -1339 && a.hi() ==  1300);
        a = Lohi {-103,  100}; b = Lohi {-13,  10}; a *= b; assert(a.lo() == -1300 && a.hi() ==  1339);
        a = Lohi {-100,  103}; b = Lohi {-20, -10}; a *= b; assert(a.lo() == -2060 && a.hi() ==  2000);
        a = Lohi {-200, -100}; b = Lohi { 10,  20}; a *= b; assert(a.lo() == -4000 && a.hi() == -1000);
        a = Lohi {-200, -100}; b = Lohi {-10,  13}; a *= b; assert(a.lo() == -2600 && a.hi() ==  2000);
        a = Lohi {-200, -100}; b = Lohi {-20, -10}; a *= b; assert(a.lo() ==  1000 && a.hi() ==  4000);
        a = Lohi {   0,  200}; b = Lohi { 10,  20}; a *= b; assert(a.lo() ==     0 && a.hi() ==  4000);
        a = Lohi {   0,  200}; b = Lohi {-10,  13}; a *= b; assert(a.lo() == -2000 && a.hi() ==  2600);
        a = Lohi {   0,  200}; b = Lohi {-20, -10}; a *= b; assert(a.lo() == -4000 && a.hi() ==     0);
        a = Lohi {   0,  103}; b = Lohi { 10,  20}; a *= b; assert(a.lo() ==     0 && a.hi() ==  2060);
        a = Lohi {   0,  103}; b = Lohi {-10,  13}; a *= b; assert(a.lo() == -1030 && a.hi() ==  1339);
        a = Lohi {   0,  103}; b = Lohi {-13,  10}; a *= b; assert(a.lo() == -1339 && a.hi() ==  1030);
        a = Lohi {   0,  100}; b = Lohi {-10,  13}; a *= b; assert(a.lo() == -1000 && a.hi() ==  1300);
        a = Lohi {   0,  100}; b = Lohi {-13,  10}; a *= b; assert(a.lo() == -1300 && a.hi() ==  1000);
        a = Lohi {   0,  103}; b = Lohi {-20, -10}; a *= b; assert(a.lo() == -2060 && a.hi() ==     0);
        a = Lohi {-100,    0}; b = Lohi { 10,  20}; a *= b; assert(a.lo() == -2000 && a.hi() ==     0);
        a = Lohi {-100,    0}; b = Lohi {-10,  13}; a *= b; assert(a.lo() == -1300 && a.hi() ==  1000);
        a = Lohi {-100,    0}; b = Lohi {-13,  10}; a *= b; assert(a.lo() == -1000 && a.hi() ==  1300);
        a = Lohi {-103,    0}; b = Lohi {-10,  13}; a *= b; assert(a.lo() == -1339 && a.hi() ==  1030);
        a = Lohi {-103,    0}; b = Lohi {-13,  10}; a *= b; assert(a.lo() == -1030 && a.hi() ==  1339);
        a = Lohi {-100,    0}; b = Lohi {-20, -10}; a *= b; assert(a.lo() ==     0 && a.hi() ==  2000);
        a = Lohi {-200,    0}; b = Lohi { 10,  20}; a *= b; assert(a.lo() == -4000 && a.hi() ==     0);
        a = Lohi {-200,    0}; b = Lohi {-10,  13}; a *= b; assert(a.lo() == -2600 && a.hi() ==  2000);
        a = Lohi {-200,    0}; b = Lohi {-20, -10}; a *= b; assert(a.lo() ==     0 && a.hi() ==  4000);
        a = Lohi { 100,  200}; b = Lohi {  0,  20}; a *= b; assert(a.lo() ==     0 && a.hi() ==  4000);
        a = Lohi { 100,  200}; b = Lohi {  0,  13}; a *= b; assert(a.lo() ==     0 && a.hi() ==  2600);
        a = Lohi {-100,  103}; b = Lohi {  0,  20}; a *= b; assert(a.lo() == -2000 && a.hi() ==  2060);
        a = Lohi {-100,  103}; b = Lohi {  0,  13}; a *= b; assert(a.lo() == -1300 && a.hi() ==  1339);
        a = Lohi {-100,  103}; b = Lohi {  0,  10}; a *= b; assert(a.lo() == -1000 && a.hi() ==  1030);
        a = Lohi {-103,  100}; b = Lohi {  0,  13}; a *= b; assert(a.lo() == -1339 && a.hi() ==  1300);
        a = Lohi {-103,  100}; b = Lohi {  0,  10}; a *= b; assert(a.lo() == -1030 && a.hi() ==  1000);
        a = Lohi {-200, -100}; b = Lohi {  0,  20}; a *= b; assert(a.lo() == -4000 && a.hi() ==     0);
        a = Lohi {-200, -100}; b = Lohi {  0,  13}; a *= b; assert(a.lo() == -2600 && a.hi() ==     0);
        a = Lohi { 100,  200}; b = Lohi {-10,   0}; a *= b; assert(a.lo() == -2000 && a.hi() ==     0);
        a = Lohi { 100,  200}; b = Lohi {-20,   0}; a *= b; assert(a.lo() == -4000 && a.hi() ==     0);
        a = Lohi {-100,  103}; b = Lohi {-10,   0}; a *= b; assert(a.lo() == -1030 && a.hi() ==  1000);
        a = Lohi {-100,  103}; b = Lohi {-13,   0}; a *= b; assert(a.lo() == -1339 && a.hi() ==  1300);
        a = Lohi {-103,  100}; b = Lohi {-10,   0}; a *= b; assert(a.lo() == -1000 && a.hi() ==  1030);
        a = Lohi {-103,  100}; b = Lohi {-13,   0}; a *= b; assert(a.lo() == -1300 && a.hi() ==  1339);
        a = Lohi {-100,  103}; b = Lohi {-20,   0}; a *= b; assert(a.lo() == -2060 && a.hi() ==  2000);
        a = Lohi {-200, -100}; b = Lohi {-10,   0}; a *= b; assert(a.lo() ==     0 && a.hi() ==  2000);
        a = Lohi {-200, -100}; b = Lohi {-20,   0}; a *= b; assert(a.lo() ==     0 && a.hi() ==  4000);
        a = Lohi {   0,  200}; b = Lohi {  0,  20}; a *= b; assert(a.lo() ==     0 && a.hi() ==  4000);
        a = Lohi {   0,  200}; b = Lohi {-10,   0}; a *= b; assert(a.lo() == -2000 && a.hi() ==     0);
        a = Lohi {-100,    0}; b = Lohi {  0,  20}; a *= b; assert(a.lo() == -2000 && a.hi() ==     0);
        a = Lohi {-100,    0}; b = Lohi {-10,   0}; a *= b; assert(a.lo() ==     0 && a.hi() ==  1000);
        a = Lohi {   0,    0}; b = Lohi { 10,  20}; a *= b; assert(a.lo() ==     0 && a.hi() ==     0);
        a = Lohi { 100,  200}; b = Lohi {  0,   0}; a *= b; assert(a.lo() ==     0 && a.hi() ==     0);
        a = Lohi {   0,    0}; b = Lohi {-10,  20}; a *= b; assert(a.lo() ==     0 && a.hi() ==     0);
        a = Lohi {-100,  200}; b = Lohi {  0,   0}; a *= b; assert(a.lo() ==     0 && a.hi() ==     0);
        a = Lohi {   0,    0}; b = Lohi {-20,  10}; a *= b; assert(a.lo() ==     0 && a.hi() ==     0);
        a = Lohi {-200,  100}; b = Lohi {  0,   0}; a *= b; assert(a.lo() ==     0 && a.hi() ==     0);
        a = Lohi {   0,    0}; b = Lohi {-20, -10}; a *= b; assert(a.lo() ==     0 && a.hi() ==     0);
        a = Lohi {-200, -100}; b = Lohi {  0,   0}; a *= b; assert(a.lo() ==     0 && a.hi() ==     0);
        a = Lohi {-100,    0}; b = Lohi {  0,   0}; a *= b; assert(a.lo() ==     0 && a.hi() ==     0);
        a = Lohi {   0,  200}; b = Lohi {  0,   0}; a *= b; assert(a.lo() ==     0 && a.hi() ==     0);
        a = Lohi {   0,    0}; b = Lohi {-10,   0}; a *= b; assert(a.lo() ==     0 && a.hi() ==     0);
        a = Lohi {   0,    0}; b = Lohi {  0,  20}; a *= b; assert(a.lo() ==     0 && a.hi() ==     0);
        a = Lohi {   0,    0}; b = Lohi {  0,   0}; a *= b; assert(a.lo() ==     0 && a.hi() ==     0);
        a = Lohi { 10,  20}; a *= a; assert(a.lo() ==  100 && a.hi() ==  400);
        a = Lohi {-10,  13}; a *= a; assert(a.lo() == -130 && a.hi() ==  169);
        a = Lohi {-13,  10}; a *= a; assert(a.lo() == -130 && a.hi() ==  169);
        a = Lohi {-20, -10}; a *= a; assert(a.lo() ==  100 && a.hi() ==  400);
        a = Lohi {  0,  20}; a *= a; assert(a.lo() ==    0 && a.hi() ==  400);
        a = Lohi {-10,   0}; a *= a; assert(a.lo() ==    0 && a.hi() ==  100);
        a = Lohi {  0,   0}; a *= a; assert(a.lo() ==    0 && a.hi() ==    0);
    }

    // Interval::Lohi::operator*=(const mpz_class &)
    {
        using Interval::Lohi;
        Lohi a;
        a = Lohi { 100,  200}; a *=  j; assert(a.lo() ==  1000 && a.hi() ==  2000);
        a = Lohi {-100,  103}; a *=  j; assert(a.lo() == -1000 && a.hi() ==  1030);
        a = Lohi {-103,  100}; a *=  j; assert(a.lo() == -1030 && a.hi() ==  1000);
        a = Lohi {-200, -100}; a *=  j; assert(a.lo() == -2000 && a.hi() == -1000);
        a = Lohi { 100,  200}; a *= -j; assert(a.lo() == -2000 && a.hi() == -1000);
        a = Lohi {-100,  103}; a *= -j; assert(a.lo() == -1030 && a.hi() ==  1000);
        a = Lohi {-103,  100}; a *= -j; assert(a.lo() == -1000 && a.hi() ==  1030);
        a = Lohi {-200, -100}; a *= -j; assert(a.lo() ==  1000 && a.hi() ==  2000);
        a = Lohi {   0,  200}; a *=  j; assert(a.lo() ==     0 && a.hi() ==  2000);
        a = Lohi {-103,    0}; a *=  j; assert(a.lo() == -1030 && a.hi() ==     0);
        a = Lohi {   0,  200}; a *= -j; assert(a.lo() == -2000 && a.hi() ==     0);
        a = Lohi {-103,    0}; a *= -j; assert(a.lo() ==     0 && a.hi() ==  1030);
        a = Lohi {   0,    0}; a *=  j; assert(a.lo() ==     0 && a.hi() ==     0);
        a = Lohi {   0,    0}; a *= -j; assert(a.lo() ==     0 && a.hi() ==     0);
        a = Lohi { 100,  200}; a *=  a.lo(); assert(a.lo() == 10000 && a.hi() == 20000);
        a = Lohi {-100,  103}; a *=  a.lo(); assert(a.lo() ==-10300 && a.hi() == 10000);
        a = Lohi {-103,  100}; a *=  a.lo(); assert(a.lo() ==-10300 && a.hi() == 10609);
        a = Lohi {-200, -100}; a *=  a.lo(); assert(a.lo() == 20000 && a.hi() == 40000);
        a = Lohi {   0,  200}; a *=  a.lo(); assert(a.lo() ==     0 && a.hi() ==     0);
        a = Lohi {-103,    0}; a *=  a.lo(); assert(a.lo() ==     0 && a.hi() == 10609);
        a = Lohi {   0,    0}; a *=  a.lo(); assert(a.lo() ==     0 && a.hi() ==     0);
        a = Lohi { 100,  200}; a *=  a.hi(); assert(a.lo() == 20000 && a.hi() == 40000);
        a = Lohi {-100,  103}; a *=  a.hi(); assert(a.lo() ==-10300 && a.hi() == 10609);
        a = Lohi {-103,  100}; a *=  a.hi(); assert(a.lo() ==-10300 && a.hi() == 10000);
        a = Lohi {-200, -100}; a *=  a.hi(); assert(a.lo() == 10000 && a.hi() == 20000);
        a = Lohi {   0,  200}; a *=  a.hi(); assert(a.lo() ==     0 && a.hi() == 40000);
        a = Lohi {-103,    0}; a *=  a.hi(); assert(a.lo() ==     0 && a.hi() ==     0);
        a = Lohi {   0,    0}; a *=  a.hi(); assert(a.lo() ==     0 && a.hi() ==     0);
    }

    // Interval::Lohi::operator*=(long)
    {
        using Interval::Lohi;
        Lohi a;
        a = Lohi { 100,  200}; a *=  k; assert(a.lo() ==   400 && a.hi() ==   800);
        a = Lohi {-100,  103}; a *=  k; assert(a.lo() ==  -400 && a.hi() ==   412);
        a = Lohi {-103,  100}; a *=  k; assert(a.lo() ==  -412 && a.hi() ==   400);
        a = Lohi {-200, -100}; a *=  k; assert(a.lo() ==  -800 && a.hi() ==  -400);
        a = Lohi { 100,  200}; a *= -k; assert(a.lo() ==  -800 && a.hi() ==  -400);
        a = Lohi {-100,  103}; a *= -k; assert(a.lo() ==  -412 && a.hi() ==   400);
        a = Lohi {-103,  100}; a *= -k; assert(a.lo() ==  -400 && a.hi() ==   412);
        a = Lohi {-200, -100}; a *= -k; assert(a.lo() ==   400 && a.hi() ==   800);
        a = Lohi {   0,  200}; a *=  k; assert(a.lo() ==     0 && a.hi() ==   800);
        a = Lohi {-103,    0}; a *=  k; assert(a.lo() ==  -412 && a.hi() ==     0);
        a = Lohi {   0,  200}; a *= -k; assert(a.lo() ==  -800 && a.hi() ==     0);
        a = Lohi {-103,    0}; a *= -k; assert(a.lo() ==     0 && a.hi() ==   412);
        a = Lohi {   0,    0}; a *=  k; assert(a.lo() ==     0 && a.hi() ==     0);
        a = Lohi {   0,    0}; a *= -k; assert(a.lo() ==     0 && a.hi() ==     0);
    }

    // Interval::Lohi::operator/=(const mpz_class &)
    {
        using Interval::Lohi;
        Lohi a;
        a = Lohi {-18, -18}; a /= mpz_class {+3}; assert(a.lo() == -6 && a.hi() == -6);
        a = Lohi {-18, -16}; a /= mpz_class {+3}; assert(a.lo() == -6 && a.hi() == -5);
        a = Lohi {-18, -14}; a /= mpz_class {+3}; assert(a.lo() == -6 && a.hi() == -4);
        a = Lohi {-18, -12}; a /= mpz_class {+3}; assert(a.lo() == -6 && a.hi() == -4);
        a = Lohi {-18, + 0}; a /= mpz_class {+3}; assert(a.lo() == -6 && a.hi() == +0);
        a = Lohi {-18, +12}; a /= mpz_class {+3}; assert(a.lo() == -6 && a.hi() == +4);
        a = Lohi {-18, +14}; a /= mpz_class {+3}; assert(a.lo() == -6 && a.hi() == +5);
        a = Lohi {-18, +16}; a /= mpz_class {+3}; assert(a.lo() == -6 && a.hi() == +6);
        a = Lohi {-18, +18}; a /= mpz_class {+3}; assert(a.lo() == -6 && a.hi() == +6);
        a = Lohi {-16, -16}; a /= mpz_class {+3}; assert(a.lo() == -6 && a.hi() == -5);
        a = Lohi {-16, -14}; a /= mpz_class {+3}; assert(a.lo() == -6 && a.hi() == -4);
        a = Lohi {-16, -12}; a /= mpz_class {+3}; assert(a.lo() == -6 && a.hi() == -4);
        a = Lohi {-16, + 0}; a /= mpz_class {+3}; assert(a.lo() == -6 && a.hi() == +0);
        a = Lohi {-16, +12}; a /= mpz_class {+3}; assert(a.lo() == -6 && a.hi() == +4);
        a = Lohi {-16, +14}; a /= mpz_class {+3}; assert(a.lo() == -6 && a.hi() == +5);
        a = Lohi {-16, +16}; a /= mpz_class {+3}; assert(a.lo() == -6 && a.hi() == +6);
        a = Lohi {-16, +18}; a /= mpz_class {+3}; assert(a.lo() == -6 && a.hi() == +6);
        a = Lohi {-14, -14}; a /= mpz_class {+3}; assert(a.lo() == -5 && a.hi() == -4);
        a = Lohi {-14, -12}; a /= mpz_class {+3}; assert(a.lo() == -5 && a.hi() == -4);
        a = Lohi {-14, + 0}; a /= mpz_class {+3}; assert(a.lo() == -5 && a.hi() == +0);
        a = Lohi {-14, +12}; a /= mpz_class {+3}; assert(a.lo() == -5 && a.hi() == +4);
        a = Lohi {-14, +14}; a /= mpz_class {+3}; assert(a.lo() == -5 && a.hi() == +5);
        a = Lohi {-14, +16}; a /= mpz_class {+3}; assert(a.lo() == -5 && a.hi() == +6);
        a = Lohi {-14, +18}; a /= mpz_class {+3}; assert(a.lo() == -5 && a.hi() == +6);
        a = Lohi {-12, -12}; a /= mpz_class {+3}; assert(a.lo() == -4 && a.hi() == -4);
        a = Lohi {-12, + 0}; a /= mpz_class {+3}; assert(a.lo() == -4 && a.hi() == +0);
        a = Lohi {-12, +12}; a /= mpz_class {+3}; assert(a.lo() == -4 && a.hi() == +4);
        a = Lohi {-12, +14}; a /= mpz_class {+3}; assert(a.lo() == -4 && a.hi() == +5);
        a = Lohi {-12, +16}; a /= mpz_class {+3}; assert(a.lo() == -4 && a.hi() == +6);
        a = Lohi {-12, +18}; a /= mpz_class {+3}; assert(a.lo() == -4 && a.hi() == +6);
        a = Lohi {+ 0, + 0}; a /= mpz_class {+3}; assert(a.lo() == +0 && a.hi() == +0);
        a = Lohi {+ 0, +12}; a /= mpz_class {+3}; assert(a.lo() == +0 && a.hi() == +4);
        a = Lohi {+ 0, +14}; a /= mpz_class {+3}; assert(a.lo() == +0 && a.hi() == +5);
        a = Lohi {+ 0, +16}; a /= mpz_class {+3}; assert(a.lo() == +0 && a.hi() == +6);
        a = Lohi {+ 0, +18}; a /= mpz_class {+3}; assert(a.lo() == +0 && a.hi() == +6);
        a = Lohi {+12, +12}; a /= mpz_class {+3}; assert(a.lo() == +4 && a.hi() == +4);
        a = Lohi {+12, +14}; a /= mpz_class {+3}; assert(a.lo() == +4 && a.hi() == +5);
        a = Lohi {+12, +16}; a /= mpz_class {+3}; assert(a.lo() == +4 && a.hi() == +6);
        a = Lohi {+12, +18}; a /= mpz_class {+3}; assert(a.lo() == +4 && a.hi() == +6);
        a = Lohi {+14, +14}; a /= mpz_class {+3}; assert(a.lo() == +4 && a.hi() == +5);
        a = Lohi {+14, +16}; a /= mpz_class {+3}; assert(a.lo() == +4 && a.hi() == +6);
        a = Lohi {+14, +18}; a /= mpz_class {+3}; assert(a.lo() == +4 && a.hi() == +6);
        a = Lohi {+16, +16}; a /= mpz_class {+3}; assert(a.lo() == +5 && a.hi() == +6);
        a = Lohi {+16, +18}; a /= mpz_class {+3}; assert(a.lo() == +5 && a.hi() == +6);
        a = Lohi {+18, +18}; a /= mpz_class {+3}; assert(a.lo() == +6 && a.hi() == +6);
        a = Lohi {-18, -18}; a /= mpz_class {-3}; assert(a.hi() == +6 && a.lo() == +6);
        a = Lohi {-18, -16}; a /= mpz_class {-3}; assert(a.hi() == +6 && a.lo() == +5);
        a = Lohi {-18, -14}; a /= mpz_class {-3}; assert(a.hi() == +6 && a.lo() == +4);
        a = Lohi {-18, -12}; a /= mpz_class {-3}; assert(a.hi() == +6 && a.lo() == +4);
        a = Lohi {-18, + 0}; a /= mpz_class {-3}; assert(a.hi() == +6 && a.lo() == +0);
        a = Lohi {-18, +12}; a /= mpz_class {-3}; assert(a.hi() == +6 && a.lo() == -4);
        a = Lohi {-18, +14}; a /= mpz_class {-3}; assert(a.hi() == +6 && a.lo() == -5);
        a = Lohi {-18, +16}; a /= mpz_class {-3}; assert(a.hi() == +6 && a.lo() == -6);
        a = Lohi {-18, +18}; a /= mpz_class {-3}; assert(a.hi() == +6 && a.lo() == -6);
        a = Lohi {-16, -16}; a /= mpz_class {-3}; assert(a.hi() == +6 && a.lo() == +5);
        a = Lohi {-16, -14}; a /= mpz_class {-3}; assert(a.hi() == +6 && a.lo() == +4);
        a = Lohi {-16, -12}; a /= mpz_class {-3}; assert(a.hi() == +6 && a.lo() == +4);
        a = Lohi {-16, + 0}; a /= mpz_class {-3}; assert(a.hi() == +6 && a.lo() == +0);
        a = Lohi {-16, +12}; a /= mpz_class {-3}; assert(a.hi() == +6 && a.lo() == -4);
        a = Lohi {-16, +14}; a /= mpz_class {-3}; assert(a.hi() == +6 && a.lo() == -5);
        a = Lohi {-16, +16}; a /= mpz_class {-3}; assert(a.hi() == +6 && a.lo() == -6);
        a = Lohi {-16, +18}; a /= mpz_class {-3}; assert(a.hi() == +6 && a.lo() == -6);
        a = Lohi {-14, -14}; a /= mpz_class {-3}; assert(a.hi() == +5 && a.lo() == +4);
        a = Lohi {-14, -12}; a /= mpz_class {-3}; assert(a.hi() == +5 && a.lo() == +4);
        a = Lohi {-14, + 0}; a /= mpz_class {-3}; assert(a.hi() == +5 && a.lo() == +0);
        a = Lohi {-14, +12}; a /= mpz_class {-3}; assert(a.hi() == +5 && a.lo() == -4);
        a = Lohi {-14, +14}; a /= mpz_class {-3}; assert(a.hi() == +5 && a.lo() == -5);
        a = Lohi {-14, +16}; a /= mpz_class {-3}; assert(a.hi() == +5 && a.lo() == -6);
        a = Lohi {-14, +18}; a /= mpz_class {-3}; assert(a.hi() == +5 && a.lo() == -6);
        a = Lohi {-12, -12}; a /= mpz_class {-3}; assert(a.hi() == +4 && a.lo() == +4);
        a = Lohi {-12, + 0}; a /= mpz_class {-3}; assert(a.hi() == +4 && a.lo() == +0);
        a = Lohi {-12, +12}; a /= mpz_class {-3}; assert(a.hi() == +4 && a.lo() == -4);
        a = Lohi {-12, +14}; a /= mpz_class {-3}; assert(a.hi() == +4 && a.lo() == -5);
        a = Lohi {-12, +16}; a /= mpz_class {-3}; assert(a.hi() == +4 && a.lo() == -6);
        a = Lohi {-12, +18}; a /= mpz_class {-3}; assert(a.hi() == +4 && a.lo() == -6);
        a = Lohi {+ 0, + 0}; a /= mpz_class {-3}; assert(a.hi() == +0 && a.lo() == +0);
        a = Lohi {+ 0, +12}; a /= mpz_class {-3}; assert(a.hi() == +0 && a.lo() == -4);
        a = Lohi {+ 0, +14}; a /= mpz_class {-3}; assert(a.hi() == +0 && a.lo() == -5);
        a = Lohi {+ 0, +16}; a /= mpz_class {-3}; assert(a.hi() == +0 && a.lo() == -6);
        a = Lohi {+ 0, +18}; a /= mpz_class {-3}; assert(a.hi() == +0 && a.lo() == -6);
        a = Lohi {+12, +12}; a /= mpz_class {-3}; assert(a.hi() == -4 && a.lo() == -4);
        a = Lohi {+12, +14}; a /= mpz_class {-3}; assert(a.hi() == -4 && a.lo() == -5);
        a = Lohi {+12, +16}; a /= mpz_class {-3}; assert(a.hi() == -4 && a.lo() == -6);
        a = Lohi {+12, +18}; a /= mpz_class {-3}; assert(a.hi() == -4 && a.lo() == -6);
        a = Lohi {+14, +14}; a /= mpz_class {-3}; assert(a.hi() == -4 && a.lo() == -5);
        a = Lohi {+14, +16}; a /= mpz_class {-3}; assert(a.hi() == -4 && a.lo() == -6);
        a = Lohi {+14, +18}; a /= mpz_class {-3}; assert(a.hi() == -4 && a.lo() == -6);
        a = Lohi {+16, +16}; a /= mpz_class {-3}; assert(a.hi() == -5 && a.lo() == -6);
        a = Lohi {+16, +18}; a /= mpz_class {-3}; assert(a.hi() == -5 && a.lo() == -6);
        a = Lohi {+18, +18}; a /= mpz_class {-3}; assert(a.hi() == -6 && a.lo() == -6);
        a = Lohi {+36, +47}; a /= mpz_class {+5}; assert(a.lo() == +7 && a.hi() ==+10);
        a = Lohi {+36, +47}; a /= mpz_class {-5}; assert(a.lo() ==-10 && a.hi() == -7);
        a = Lohi {-47, -36}; a /= mpz_class {+5}; assert(a.lo() ==-10 && a.hi() == -7);
        a = Lohi {-47, -36}; a /= mpz_class {-5}; assert(a.lo() == +7 && a.hi() ==+10);
        a = Lohi {+36, +75}; a /= a.lo(); assert(a.lo() == +1 && a.hi() == +3);
        a = Lohi {+36, +75}; a /= a.hi(); assert(a.lo() == +0 && a.hi() == +1);
        a = Lohi {-75, -36}; a /= a.lo(); assert(a.lo() == +0 && a.hi() == +1);
        a = Lohi {-75, -36}; a /= a.hi(); assert(a.lo() == +1 && a.hi() == +3);
        a = Lohi {-14, +17}; a /= mpz_class {+1}; assert(a.lo() == -14 && a.hi() == +17);
        a = Lohi {-14, +17}; a /= mpz_class {-1}; assert(a.lo() == -17 && a.hi() == +14);
    }

    // Interval::Lohi::operator/=(long)
    {
        using Interval::Lohi;
        Lohi a;
        a = Lohi {-18, -18}; a /= long {+3}; assert(a.lo() == -6 && a.hi() == -6);
        a = Lohi {-18, -16}; a /= long {+3}; assert(a.lo() == -6 && a.hi() == -5);
        a = Lohi {-18, -14}; a /= long {+3}; assert(a.lo() == -6 && a.hi() == -4);
        a = Lohi {-18, -12}; a /= long {+3}; assert(a.lo() == -6 && a.hi() == -4);
        a = Lohi {-18, + 0}; a /= long {+3}; assert(a.lo() == -6 && a.hi() == +0);
        a = Lohi {-18, +12}; a /= long {+3}; assert(a.lo() == -6 && a.hi() == +4);
        a = Lohi {-18, +14}; a /= long {+3}; assert(a.lo() == -6 && a.hi() == +5);
        a = Lohi {-18, +16}; a /= long {+3}; assert(a.lo() == -6 && a.hi() == +6);
        a = Lohi {-18, +18}; a /= long {+3}; assert(a.lo() == -6 && a.hi() == +6);
        a = Lohi {-16, -16}; a /= long {+3}; assert(a.lo() == -6 && a.hi() == -5);
        a = Lohi {-16, -14}; a /= long {+3}; assert(a.lo() == -6 && a.hi() == -4);
        a = Lohi {-16, -12}; a /= long {+3}; assert(a.lo() == -6 && a.hi() == -4);
        a = Lohi {-16, + 0}; a /= long {+3}; assert(a.lo() == -6 && a.hi() == +0);
        a = Lohi {-16, +12}; a /= long {+3}; assert(a.lo() == -6 && a.hi() == +4);
        a = Lohi {-16, +14}; a /= long {+3}; assert(a.lo() == -6 && a.hi() == +5);
        a = Lohi {-16, +16}; a /= long {+3}; assert(a.lo() == -6 && a.hi() == +6);
        a = Lohi {-16, +18}; a /= long {+3}; assert(a.lo() == -6 && a.hi() == +6);
        a = Lohi {-14, -14}; a /= long {+3}; assert(a.lo() == -5 && a.hi() == -4);
        a = Lohi {-14, -12}; a /= long {+3}; assert(a.lo() == -5 && a.hi() == -4);
        a = Lohi {-14, + 0}; a /= long {+3}; assert(a.lo() == -5 && a.hi() == +0);
        a = Lohi {-14, +12}; a /= long {+3}; assert(a.lo() == -5 && a.hi() == +4);
        a = Lohi {-14, +14}; a /= long {+3}; assert(a.lo() == -5 && a.hi() == +5);
        a = Lohi {-14, +16}; a /= long {+3}; assert(a.lo() == -5 && a.hi() == +6);
        a = Lohi {-14, +18}; a /= long {+3}; assert(a.lo() == -5 && a.hi() == +6);
        a = Lohi {-12, -12}; a /= long {+3}; assert(a.lo() == -4 && a.hi() == -4);
        a = Lohi {-12, + 0}; a /= long {+3}; assert(a.lo() == -4 && a.hi() == +0);
        a = Lohi {-12, +12}; a /= long {+3}; assert(a.lo() == -4 && a.hi() == +4);
        a = Lohi {-12, +14}; a /= long {+3}; assert(a.lo() == -4 && a.hi() == +5);
        a = Lohi {-12, +16}; a /= long {+3}; assert(a.lo() == -4 && a.hi() == +6);
        a = Lohi {-12, +18}; a /= long {+3}; assert(a.lo() == -4 && a.hi() == +6);
        a = Lohi {+ 0, + 0}; a /= long {+3}; assert(a.lo() == +0 && a.hi() == +0);
        a = Lohi {+ 0, +12}; a /= long {+3}; assert(a.lo() == +0 && a.hi() == +4);
        a = Lohi {+ 0, +14}; a /= long {+3}; assert(a.lo() == +0 && a.hi() == +5);
        a = Lohi {+ 0, +16}; a /= long {+3}; assert(a.lo() == +0 && a.hi() == +6);
        a = Lohi {+ 0, +18}; a /= long {+3}; assert(a.lo() == +0 && a.hi() == +6);
        a = Lohi {+12, +12}; a /= long {+3}; assert(a.lo() == +4 && a.hi() == +4);
        a = Lohi {+12, +14}; a /= long {+3}; assert(a.lo() == +4 && a.hi() == +5);
        a = Lohi {+12, +16}; a /= long {+3}; assert(a.lo() == +4 && a.hi() == +6);
        a = Lohi {+12, +18}; a /= long {+3}; assert(a.lo() == +4 && a.hi() == +6);
        a = Lohi {+14, +14}; a /= long {+3}; assert(a.lo() == +4 && a.hi() == +5);
        a = Lohi {+14, +16}; a /= long {+3}; assert(a.lo() == +4 && a.hi() == +6);
        a = Lohi {+14, +18}; a /= long {+3}; assert(a.lo() == +4 && a.hi() == +6);
        a = Lohi {+16, +16}; a /= long {+3}; assert(a.lo() == +5 && a.hi() == +6);
        a = Lohi {+16, +18}; a /= long {+3}; assert(a.lo() == +5 && a.hi() == +6);
        a = Lohi {+18, +18}; a /= long {+3}; assert(a.lo() == +6 && a.hi() == +6);
        a = Lohi {-18, -18}; a /= long {-3}; assert(a.hi() == +6 && a.lo() == +6);
        a = Lohi {-18, -16}; a /= long {-3}; assert(a.hi() == +6 && a.lo() == +5);
        a = Lohi {-18, -14}; a /= long {-3}; assert(a.hi() == +6 && a.lo() == +4);
        a = Lohi {-18, -12}; a /= long {-3}; assert(a.hi() == +6 && a.lo() == +4);
        a = Lohi {-18, + 0}; a /= long {-3}; assert(a.hi() == +6 && a.lo() == +0);
        a = Lohi {-18, +12}; a /= long {-3}; assert(a.hi() == +6 && a.lo() == -4);
        a = Lohi {-18, +14}; a /= long {-3}; assert(a.hi() == +6 && a.lo() == -5);
        a = Lohi {-18, +16}; a /= long {-3}; assert(a.hi() == +6 && a.lo() == -6);
        a = Lohi {-18, +18}; a /= long {-3}; assert(a.hi() == +6 && a.lo() == -6);
        a = Lohi {-16, -16}; a /= long {-3}; assert(a.hi() == +6 && a.lo() == +5);
        a = Lohi {-16, -14}; a /= long {-3}; assert(a.hi() == +6 && a.lo() == +4);
        a = Lohi {-16, -12}; a /= long {-3}; assert(a.hi() == +6 && a.lo() == +4);
        a = Lohi {-16, + 0}; a /= long {-3}; assert(a.hi() == +6 && a.lo() == +0);
        a = Lohi {-16, +12}; a /= long {-3}; assert(a.hi() == +6 && a.lo() == -4);
        a = Lohi {-16, +14}; a /= long {-3}; assert(a.hi() == +6 && a.lo() == -5);
        a = Lohi {-16, +16}; a /= long {-3}; assert(a.hi() == +6 && a.lo() == -6);
        a = Lohi {-16, +18}; a /= long {-3}; assert(a.hi() == +6 && a.lo() == -6);
        a = Lohi {-14, -14}; a /= long {-3}; assert(a.hi() == +5 && a.lo() == +4);
        a = Lohi {-14, -12}; a /= long {-3}; assert(a.hi() == +5 && a.lo() == +4);
        a = Lohi {-14, + 0}; a /= long {-3}; assert(a.hi() == +5 && a.lo() == +0);
        a = Lohi {-14, +12}; a /= long {-3}; assert(a.hi() == +5 && a.lo() == -4);
        a = Lohi {-14, +14}; a /= long {-3}; assert(a.hi() == +5 && a.lo() == -5);
        a = Lohi {-14, +16}; a /= long {-3}; assert(a.hi() == +5 && a.lo() == -6);
        a = Lohi {-14, +18}; a /= long {-3}; assert(a.hi() == +5 && a.lo() == -6);
        a = Lohi {-12, -12}; a /= long {-3}; assert(a.hi() == +4 && a.lo() == +4);
        a = Lohi {-12, + 0}; a /= long {-3}; assert(a.hi() == +4 && a.lo() == +0);
        a = Lohi {-12, +12}; a /= long {-3}; assert(a.hi() == +4 && a.lo() == -4);
        a = Lohi {-12, +14}; a /= long {-3}; assert(a.hi() == +4 && a.lo() == -5);
        a = Lohi {-12, +16}; a /= long {-3}; assert(a.hi() == +4 && a.lo() == -6);
        a = Lohi {-12, +18}; a /= long {-3}; assert(a.hi() == +4 && a.lo() == -6);
        a = Lohi {+ 0, + 0}; a /= long {-3}; assert(a.hi() == +0 && a.lo() == +0);
        a = Lohi {+ 0, +12}; a /= long {-3}; assert(a.hi() == +0 && a.lo() == -4);
        a = Lohi {+ 0, +14}; a /= long {-3}; assert(a.hi() == +0 && a.lo() == -5);
        a = Lohi {+ 0, +16}; a /= long {-3}; assert(a.hi() == +0 && a.lo() == -6);
        a = Lohi {+ 0, +18}; a /= long {-3}; assert(a.hi() == +0 && a.lo() == -6);
        a = Lohi {+12, +12}; a /= long {-3}; assert(a.hi() == -4 && a.lo() == -4);
        a = Lohi {+12, +14}; a /= long {-3}; assert(a.hi() == -4 && a.lo() == -5);
        a = Lohi {+12, +16}; a /= long {-3}; assert(a.hi() == -4 && a.lo() == -6);
        a = Lohi {+12, +18}; a /= long {-3}; assert(a.hi() == -4 && a.lo() == -6);
        a = Lohi {+14, +14}; a /= long {-3}; assert(a.hi() == -4 && a.lo() == -5);
        a = Lohi {+14, +16}; a /= long {-3}; assert(a.hi() == -4 && a.lo() == -6);
        a = Lohi {+14, +18}; a /= long {-3}; assert(a.hi() == -4 && a.lo() == -6);
        a = Lohi {+16, +16}; a /= long {-3}; assert(a.hi() == -5 && a.lo() == -6);
        a = Lohi {+16, +18}; a /= long {-3}; assert(a.hi() == -5 && a.lo() == -6);
        a = Lohi {+18, +18}; a /= long {-3}; assert(a.hi() == -6 && a.lo() == -6);
        a = Lohi {+36, +47}; a /= long {+5}; assert(a.lo() == +7 && a.hi() ==+10);
        a = Lohi {+36, +47}; a /= long {-5}; assert(a.lo() ==-10 && a.hi() == -7);
        a = Lohi {-47, -36}; a /= long {+5}; assert(a.lo() ==-10 && a.hi() == -7);
        a = Lohi {-47, -36}; a /= long {-5}; assert(a.lo() == +7 && a.hi() ==+10);
        a = Lohi {+36, +47}; a /= +5; assert(a.lo() == +7 && a.hi() ==+10);
        a = Lohi {+36, +47}; a /= -5; assert(a.lo() ==-10 && a.hi() == -7);
        a = Lohi {-47, -36}; a /= +5; assert(a.lo() ==-10 && a.hi() == -7);
        a = Lohi {-47, -36}; a /= -5; assert(a.lo() == +7 && a.hi() ==+10);
        a = Lohi {-14, +17}; a /= long {+1}; assert(a.lo() == -14 && a.hi() == +17);
        a = Lohi {-14, +17}; a /= long {-1}; assert(a.lo() == -17 && a.hi() == +14);
    }

    // Interval::Lohi::operator<<=(mp_bitcnt_t)
    // Interval::Lohi::operator>>=(mp_bitcnt_t)
    {
        using Interval::Lohi;
        Lohi a;
        a = Lohi{+26, +32}; a <<= 0; assert(a.lo() ==  +26 && a.hi() ==  +32);
        a = Lohi{+26, +32}; a <<= 1; assert(a.lo() ==  +52 && a.hi() ==  +64);
        a = Lohi{+26, +32}; a <<= 2; assert(a.lo() == +104 && a.hi() == +128);
        a = Lohi{+26, +32}; a <<= 3; assert(a.lo() == +208 && a.hi() == +256);
        a = Lohi{+26, +32}; a >>= 0; assert(a.lo() ==  +26 && a.hi() ==  +32);
        a = Lohi{+26, +32}; a >>= 1; assert(a.lo() ==  +13 && a.hi() ==  +16);
        a = Lohi{+26, +32}; a >>= 2; assert(a.lo() ==   +6 && a.hi() ==   +8);
        a = Lohi{+26, +32}; a >>= 3; assert(a.lo() ==   +3 && a.hi() ==   +4);
        a = Lohi{+26, +32}; a >>= 4; assert(a.lo() ==   +1 && a.hi() ==   +2);
        a = Lohi{+26, +32}; a >>= 5; assert(a.lo() ==   +0 && a.hi() ==   +1);
        a = Lohi{+26, +32}; a >>= 6; assert(a.lo() ==   +0 && a.hi() ==   +1);
        a = Lohi{+26, +32}; a >>= 7; assert(a.lo() ==   +0 && a.hi() ==   +1);
        a = Lohi{+21, +33}; a <<= 0; assert(a.lo() ==  +21 && a.hi() ==  +33);
        a = Lohi{+21, +33}; a <<= 1; assert(a.lo() ==  +42 && a.hi() ==  +66);
        a = Lohi{+21, +33}; a <<= 2; assert(a.lo() ==  +84 && a.hi() == +132);
        a = Lohi{+21, +33}; a <<= 3; assert(a.lo() == +168 && a.hi() == +264);
        a = Lohi{+21, +33}; a >>= 0; assert(a.lo() ==  +21 && a.hi() ==  +33);
        a = Lohi{+21, +33}; a >>= 1; assert(a.lo() ==  +10 && a.hi() ==  +17);
        a = Lohi{+21, +33}; a >>= 2; assert(a.lo() ==   +5 && a.hi() ==   +9);
        a = Lohi{+21, +33}; a >>= 3; assert(a.lo() ==   +2 && a.hi() ==   +5);
        a = Lohi{+21, +33}; a >>= 4; assert(a.lo() ==   +1 && a.hi() ==   +3);
        a = Lohi{+21, +33}; a >>= 5; assert(a.lo() ==   +0 && a.hi() ==   +2);
        a = Lohi{+21, +33}; a >>= 6; assert(a.lo() ==   +0 && a.hi() ==   +1);
        a = Lohi{+21, +33}; a >>= 7; assert(a.lo() ==   +0 && a.hi() ==   +1);
        a = Lohi{-26, +32}; a <<= 0; assert(a.lo() ==  -26 && a.hi() ==  +32);
        a = Lohi{-26, +32}; a <<= 1; assert(a.lo() ==  -52 && a.hi() ==  +64);
        a = Lohi{-26, +32}; a <<= 2; assert(a.lo() == -104 && a.hi() == +128);
        a = Lohi{-26, +32}; a <<= 3; assert(a.lo() == -208 && a.hi() == +256);
        a = Lohi{-26, +32}; a >>= 0; assert(a.lo() ==  -26 && a.hi() ==  +32);
        a = Lohi{-26, +32}; a >>= 1; assert(a.lo() ==  -13 && a.hi() ==  +16);
        a = Lohi{-26, +32}; a >>= 2; assert(a.lo() ==   -7 && a.hi() ==   +8);
        a = Lohi{-26, +32}; a >>= 3; assert(a.lo() ==   -4 && a.hi() ==   +4);
        a = Lohi{-26, +32}; a >>= 4; assert(a.lo() ==   -2 && a.hi() ==   +2);
        a = Lohi{-26, +32}; a >>= 5; assert(a.lo() ==   -1 && a.hi() ==   +1);
        a = Lohi{-26, +32}; a >>= 6; assert(a.lo() ==   -1 && a.hi() ==   +1);
        a = Lohi{-26, +32}; a >>= 7; assert(a.lo() ==   -1 && a.hi() ==   +1);
        a = Lohi{-21, +33}; a <<= 0; assert(a.lo() ==  -21 && a.hi() ==  +33);
        a = Lohi{-21, +33}; a <<= 1; assert(a.lo() ==  -42 && a.hi() ==  +66);
        a = Lohi{-21, +33}; a <<= 2; assert(a.lo() ==  -84 && a.hi() == +132);
        a = Lohi{-21, +33}; a <<= 3; assert(a.lo() == -168 && a.hi() == +264);
        a = Lohi{-21, +33}; a >>= 0; assert(a.lo() ==  -21 && a.hi() ==  +33);
        a = Lohi{-21, +33}; a >>= 1; assert(a.lo() ==  -11 && a.hi() ==  +17);
        a = Lohi{-21, +33}; a >>= 2; assert(a.lo() ==   -6 && a.hi() ==   +9);
        a = Lohi{-21, +33}; a >>= 3; assert(a.lo() ==   -3 && a.hi() ==   +5);
        a = Lohi{-21, +33}; a >>= 4; assert(a.lo() ==   -2 && a.hi() ==   +3);
        a = Lohi{-21, +33}; a >>= 5; assert(a.lo() ==   -1 && a.hi() ==   +2);
        a = Lohi{-21, +33}; a >>= 6; assert(a.lo() ==   -1 && a.hi() ==   +1);
        a = Lohi{-21, +33}; a >>= 7; assert(a.lo() ==   -1 && a.hi() ==   +1);
        a = Lohi{-32, -26}; a <<= 0; assert(a.lo() ==  -32 && a.hi() ==  -26);
        a = Lohi{-32, -26}; a <<= 1; assert(a.lo() ==  -64 && a.hi() ==  -52);
        a = Lohi{-32, -26}; a <<= 2; assert(a.lo() == -128 && a.hi() == -104);
        a = Lohi{-32, -26}; a <<= 3; assert(a.lo() == -256 && a.hi() == -208);
        a = Lohi{-32, -26}; a >>= 0; assert(a.lo() ==  -32 && a.hi() ==  -26);
        a = Lohi{-32, -26}; a >>= 1; assert(a.lo() ==  -16 && a.hi() ==  -13);
        a = Lohi{-32, -26}; a >>= 2; assert(a.lo() ==   -8 && a.hi() ==   -6);
        a = Lohi{-32, -26}; a >>= 3; assert(a.lo() ==   -4 && a.hi() ==   -3);
        a = Lohi{-32, -26}; a >>= 4; assert(a.lo() ==   -2 && a.hi() ==   -1);
        a = Lohi{-32, -26}; a >>= 5; assert(a.lo() ==   -1 && a.hi() ==   +0);
        a = Lohi{-32, -26}; a >>= 6; assert(a.lo() ==   -1 && a.hi() ==   +0);
        a = Lohi{-32, -26}; a >>= 7; assert(a.lo() ==   -1 && a.hi() ==   +0);
        a = Lohi{-33, -21}; a <<= 0; assert(a.lo() ==  -33 && a.hi() ==  -21);
        a = Lohi{-33, -21}; a <<= 1; assert(a.lo() ==  -66 && a.hi() ==  -42);
        a = Lohi{-33, -21}; a <<= 2; assert(a.lo() == -132 && a.hi() ==  -84);
        a = Lohi{-33, -21}; a <<= 3; assert(a.lo() == -264 && a.hi() == -168);
        a = Lohi{-33, -21}; a >>= 0; assert(a.lo() ==  -33 && a.hi() ==  -21);
        a = Lohi{-33, -21}; a >>= 1; assert(a.lo() ==  -17 && a.hi() ==  -10);
        a = Lohi{-33, -21}; a >>= 2; assert(a.lo() ==   -9 && a.hi() ==   -5);
        a = Lohi{-33, -21}; a >>= 3; assert(a.lo() ==   -5 && a.hi() ==   -2);
        a = Lohi{-33, -21}; a >>= 4; assert(a.lo() ==   -3 && a.hi() ==   -1);
        a = Lohi{-33, -21}; a >>= 5; assert(a.lo() ==   -2 && a.hi() ==   +0);
        a = Lohi{-33, -21}; a >>= 6; assert(a.lo() ==   -1 && a.hi() ==   +0);
        a = Lohi{-33, -21}; a >>= 7; assert(a.lo() ==   -1 && a.hi() ==   +0);
        a = Lohi{-33,  +0}; a <<= 0; assert(a.lo() ==  -33 && a.hi() ==   +0);
        a = Lohi{-33,  +0}; a <<= 1; assert(a.lo() ==  -66 && a.hi() ==   +0);
        a = Lohi{-33,  +0}; a <<= 2; assert(a.lo() == -132 && a.hi() ==   +0);
        a = Lohi{-33,  +0}; a <<= 3; assert(a.lo() == -264 && a.hi() ==   +0);
        a = Lohi{-33,  +0}; a >>= 0; assert(a.lo() ==  -33 && a.hi() ==   +0);
        a = Lohi{-33,  +0}; a >>= 1; assert(a.lo() ==  -17 && a.hi() ==   +0);
        a = Lohi{-33,  +0}; a >>= 2; assert(a.lo() ==   -9 && a.hi() ==   +0);
        a = Lohi{-33,  +0}; a >>= 3; assert(a.lo() ==   -5 && a.hi() ==   +0);
        a = Lohi{-33,  +0}; a >>= 4; assert(a.lo() ==   -3 && a.hi() ==   +0);
        a = Lohi{-33,  +0}; a >>= 5; assert(a.lo() ==   -2 && a.hi() ==   +0);
        a = Lohi{-33,  +0}; a >>= 6; assert(a.lo() ==   -1 && a.hi() ==   +0);
        a = Lohi{-33,  +0}; a >>= 7; assert(a.lo() ==   -1 && a.hi() ==   +0);
        a = Lohi{ +0, +21}; a <<= 0; assert(a.lo() ==   +0 && a.hi() ==  +21);
        a = Lohi{ +0, +21}; a <<= 1; assert(a.lo() ==   +0 && a.hi() ==  +42);
        a = Lohi{ +0, +21}; a <<= 2; assert(a.lo() ==   +0 && a.hi() ==  +84);
        a = Lohi{ +0, +21}; a <<= 3; assert(a.lo() ==   +0 && a.hi() == +168);
        a = Lohi{ +0, +21}; a >>= 0; assert(a.lo() ==   +0 && a.hi() ==  +21);
        a = Lohi{ +0, +21}; a >>= 1; assert(a.lo() ==   +0 && a.hi() ==  +11);
        a = Lohi{ +0, +21}; a >>= 2; assert(a.lo() ==   +0 && a.hi() ==   +6);
        a = Lohi{ +0, +21}; a >>= 3; assert(a.lo() ==   +0 && a.hi() ==   +3);
        a = Lohi{ +0, +21}; a >>= 4; assert(a.lo() ==   +0 && a.hi() ==   +2);
        a = Lohi{ +0, +21}; a >>= 5; assert(a.lo() ==   +0 && a.hi() ==   +1);
        a = Lohi{ +0, +21}; a >>= 6; assert(a.lo() ==   +0 && a.hi() ==   +1);
        a = Lohi{ +0, +21}; a >>= 7; assert(a.lo() ==   +0 && a.hi() ==   +1);
        a = Lohi{ +0,  +0}; a <<= 0; assert(a.lo() ==   +0 && a.hi() ==   +0);
        a = Lohi{ +0,  +0}; a <<= 1; assert(a.lo() ==   +0 && a.hi() ==   +0);
        a = Lohi{ +0,  +0}; a <<= 2; assert(a.lo() ==   +0 && a.hi() ==   +0);
        a = Lohi{ +0,  +0}; a <<= 3; assert(a.lo() ==   +0 && a.hi() ==   +0);
        a = Lohi{ +0,  +0}; a >>= 0; assert(a.lo() ==   +0 && a.hi() ==   +0);
        a = Lohi{ +0,  +0}; a >>= 1; assert(a.lo() ==   +0 && a.hi() ==   +0);
        a = Lohi{ +0,  +0}; a >>= 2; assert(a.lo() ==   +0 && a.hi() ==   +0);
        a = Lohi{ +0,  +0}; a >>= 3; assert(a.lo() ==   +0 && a.hi() ==   +0);
        a = Lohi{ +0,  +0}; a >>= 4; assert(a.lo() ==   +0 && a.hi() ==   +0);
        a = Lohi{ +0,  +0}; a >>= 5; assert(a.lo() ==   +0 && a.hi() ==   +0);
        a = Lohi{ +0,  +0}; a >>= 6; assert(a.lo() ==   +0 && a.hi() ==   +0);
        a = Lohi{ +0,  +0}; a >>= 7; assert(a.lo() ==   +0 && a.hi() ==   +0);
        a = Lohi{ -1,  +1}; a >>= 1; assert(a.lo() ==   -1 && a.hi() ==   +1);
        a = Lohi{ -1,  +1}; a >>= 2; assert(a.lo() ==   -1 && a.hi() ==   +1);
        a = Lohi{ -1,  +0}; a >>= 1; assert(a.lo() ==   -1 && a.hi() ==   +0);
        a = Lohi{ -1,  +0}; a >>= 2; assert(a.lo() ==   -1 && a.hi() ==   +0);
        a = Lohi{ -0,  +1}; a >>= 1; assert(a.lo() ==   -0 && a.hi() ==   +1);
        a = Lohi{ -0,  +1}; a >>= 2; assert(a.lo() ==   -0 && a.hi() ==   +1);
    }
    // Interval::Lohi::square()
    {
        using Interval::Lohi;
        Lohi a;
        a = Lohi { 100,  200}; a.square(); assert(a.lo() == 10000 && a.hi() == 40000);
        a = Lohi {-100,  103}; a.square(); assert(a.lo() ==     0 && a.hi() == 10609);
        a = Lohi {-103,  100}; a.square(); assert(a.lo() ==     0 && a.hi() == 10609);
        a = Lohi {-200, -100}; a.square(); assert(a.lo() == 10000 && a.hi() == 40000);
        a = Lohi {   0,  200}; a.square(); assert(a.lo() ==     0 && a.hi() == 40000);
        a = Lohi {-103,    0}; a.square(); assert(a.lo() ==     0 && a.hi() == 10609);
        a = Lohi {   0,    0}; a.square(); assert(a.lo() ==     0 && a.hi() ==     0);
    }

    // Interval::Lohi::negate()
    {
        using Interval::Lohi;
        Lohi a;
        a = Lohi { 100,  200}; a.negate(); assert(a.lo() ==  -200 && a.hi() ==  -100);
        a = Lohi {-100,  103}; a.negate(); assert(a.lo() ==  -103 && a.hi() ==   100);
        a = Lohi {-103,  100}; a.negate(); assert(a.lo() ==  -100 && a.hi() ==   103);
        a = Lohi {-200, -100}; a.negate(); assert(a.lo() ==   100 && a.hi() ==   200);
        a = Lohi {   0,  200}; a.negate(); assert(a.lo() ==  -200 && a.hi() ==     0);
        a = Lohi {-103,    0}; a.negate(); assert(a.lo() ==     0 && a.hi() ==   103);
        a = Lohi {   0,    0}; a.negate(); assert(a.lo() ==     0 && a.hi() ==     0);
    }

    // Interval::Lohi::invert_against(const mpz_class &)
    {
        using Interval::Lohi;
        Lohi a;
        a = Lohi {+33, +67}; a.invert_against(+100); assert(a.lo() == +1 && a.hi() == +4);
        a = Lohi {+34, +66}; a.invert_against(+100); assert(a.lo() == +1 && a.hi() == +3);
        a = Lohi {+24, +24}; a.invert_against(+100); assert(a.lo() == +4 && a.hi() == +5);
        a = Lohi {+24, +25}; a.invert_against(+100); assert(a.lo() == +4 && a.hi() == +5);
        a = Lohi {+24, +26}; a.invert_against(+100); assert(a.lo() == +3 && a.hi() == +5);
        a = Lohi {+25, +25}; a.invert_against(+100); assert(a.lo() == +4 && a.hi() == +4);
        a = Lohi {+25, +26}; a.invert_against(+100); assert(a.lo() == +3 && a.hi() == +4);
        a = Lohi {+26, +26}; a.invert_against(+100); assert(a.lo() == +3 && a.hi() == +4);
        a = Lohi {+33, +67}; a.invert_against(-100); assert(a.hi() == -1 && a.lo() == -4);
        a = Lohi {+34, +66}; a.invert_against(-100); assert(a.hi() == -1 && a.lo() == -3);
        a = Lohi {+24, +24}; a.invert_against(-100); assert(a.hi() == -4 && a.lo() == -5);
        a = Lohi {+24, +25}; a.invert_against(-100); assert(a.hi() == -4 && a.lo() == -5);
        a = Lohi {+24, +26}; a.invert_against(-100); assert(a.hi() == -3 && a.lo() == -5);
        a = Lohi {+25, +25}; a.invert_against(-100); assert(a.hi() == -4 && a.lo() == -4);
        a = Lohi {+25, +26}; a.invert_against(-100); assert(a.hi() == -3 && a.lo() == -4);
        a = Lohi {+26, +26}; a.invert_against(-100); assert(a.hi() == -3 && a.lo() == -4);
        a = Lohi {-67, -33}; a.invert_against(+100); assert(a.hi() == -1 && a.lo() == -4);
        a = Lohi {-66, -34}; a.invert_against(+100); assert(a.hi() == -1 && a.lo() == -3);
        a = Lohi {-24, -24}; a.invert_against(+100); assert(a.hi() == -4 && a.lo() == -5);
        a = Lohi {-25, -24}; a.invert_against(+100); assert(a.hi() == -4 && a.lo() == -5);
        a = Lohi {-26, -24}; a.invert_against(+100); assert(a.hi() == -3 && a.lo() == -5);
        a = Lohi {-25, -25}; a.invert_against(+100); assert(a.hi() == -4 && a.lo() == -4);
        a = Lohi {-26, -25}; a.invert_against(+100); assert(a.hi() == -3 && a.lo() == -4);
        a = Lohi {-26, -26}; a.invert_against(+100); assert(a.hi() == -3 && a.lo() == -4);
        a = Lohi {-67, -33}; a.invert_against(-100); assert(a.lo() == +1 && a.hi() == +4);
        a = Lohi {-66, -34}; a.invert_against(-100); assert(a.lo() == +1 && a.hi() == +3);
        a = Lohi {-24, -24}; a.invert_against(-100); assert(a.lo() == +4 && a.hi() == +5);
        a = Lohi {-25, -24}; a.invert_against(-100); assert(a.lo() == +4 && a.hi() == +5);
        a = Lohi {-26, -24}; a.invert_against(-100); assert(a.lo() == +3 && a.hi() == +5);
        a = Lohi {-25, -25}; a.invert_against(-100); assert(a.lo() == +4 && a.hi() == +4);
        a = Lohi {-26, -25}; a.invert_against(-100); assert(a.lo() == +3 && a.hi() == +4);
        a = Lohi {-26, -26}; a.invert_against(-100); assert(a.lo() == +3 && a.hi() == +4);
        a = Lohi {  2,   2}; a.invert_against(   0); assert(a.lo() ==  0 && a.hi() ==  0);
        a = Lohi {  1,   2}; a.invert_against(   0); assert(a.lo() ==  0 && a.hi() ==  0);
        a = Lohi {  1,   1}; a.invert_against(   0); assert(a.lo() ==  0 && a.hi() ==  0);
        a = Lohi { -1,  -1}; a.invert_against(   0); assert(a.lo() ==  0 && a.hi() ==  0);
        a = Lohi { -2,  -1}; a.invert_against(   0); assert(a.lo() ==  0 && a.hi() ==  0);
        a = Lohi { -2,  -2}; a.invert_against(   0); assert(a.lo() ==  0 && a.hi() ==  0);
        int n_thrown {0};
        try { a = Lohi { 0,  3}; a.invert_against( 2); } catch (const Interval::Division_by_zero) { ++n_thrown; }
        try { a = Lohi { 0,  3}; a.invert_against( 1); } catch (const Interval::Division_by_zero) { ++n_thrown; }
        try { a = Lohi { 0,  3}; a.invert_against( 0); } catch (const Interval::Division_by_zero) { ++n_thrown; }
        try { a = Lohi {-1,  3}; a.invert_against( 2); } catch (const Interval::Division_by_zero) { ++n_thrown; }
        try { a = Lohi {-1,  3}; a.invert_against( 1); } catch (const Interval::Division_by_zero) { ++n_thrown; }
        try { a = Lohi {-1,  3}; a.invert_against( 0); } catch (const Interval::Division_by_zero) { ++n_thrown; }
        try { a = Lohi {-1,  0}; a.invert_against( 2); } catch (const Interval::Division_by_zero) { ++n_thrown; }
        try { a = Lohi {-1,  0}; a.invert_against( 1); } catch (const Interval::Division_by_zero) { ++n_thrown; }
        try { a = Lohi {-1,  0}; a.invert_against( 0); } catch (const Interval::Division_by_zero) { ++n_thrown; }
        assert (n_thrown == 9);
    }

    { const Interval::Lohi a {+n}; assert(a.lo() ==  27 && a.hi() ==  31); }
    { const Interval::Lohi a {-n}; assert(a.lo() == -31 && a.hi() == -27); }
    { const Interval::Lohi a {n + m}; assert(a.lo() ==  33 && a.hi() ==  38); }
    { const Interval::Lohi a {n + j}; assert(a.lo() ==  37 && a.hi() ==  41); }
    { const Interval::Lohi a {j + n}; assert(a.lo() ==  37 && a.hi() ==  41); }
    { const Interval::Lohi a {n - m}; assert(a.lo() ==  20 && a.hi() ==  25); }
    { const Interval::Lohi a {n - j}; assert(a.lo() ==  17 && a.hi() ==  21); }
    { const Interval::Lohi a {j - n}; assert(a.lo() == -21 && a.hi() == -17); }
    { const Interval::Lohi a {n * m}; assert(a.lo() == 162 && a.hi() == 217); }
    { const Interval::Lohi a {n * j}; assert(a.lo() == 270 && a.hi() == 310); }
    { const Interval::Lohi a {n * k}; assert(a.lo() == 108 && a.hi() == 124); }
    { const Interval::Lohi a {j * n}; assert(a.lo() == 270 && a.hi() == 310); }
    { const Interval::Lohi a {k * n}; assert(a.lo() == 108 && a.hi() == 124); }
    { const Interval::Lohi a {n / j}; assert(a.lo() ==   2 && a.hi() ==   4); }
    { const Interval::Lohi a {n / k}; assert(a.lo() ==   6 && a.hi() ==   8); }
    { const Interval::Lohi a {j / n}; assert(a.lo() ==   0 && a.hi() ==   1); }
    { const Interval::Lohi a {(j*j) / n}; assert(a.lo() == 3 && a.hi() == 4); }
    { const Interval::Lohi a {n << bc}; assert(a.lo() == 216 && a.hi() == 248); }
    { const Interval::Lohi a {n >> bc}; assert(a.lo() ==   3 && a.hi() ==   4); }
    { const Interval::Lohi a {square(n)}; assert(a.lo() == 729 && a.hi() == 961); }
    { const mpz_class a {Interval::square(j)}; assert(a == 100); }
    { const long      a {Interval::square(k)}; assert(a ==  16); }
    { Interval::Lohi a {n}; square(&a); assert(a.lo() == 729 && a.hi() == 961); }
    { mpz_class a {j}; Interval::square(&a); assert(a == 100); }
    { long      a {k}; Interval::square(&a); assert(a ==  16); }
    { Interval::Lohi a {n}; negate(&a); assert(a.lo() == -31 && a.hi() == -27); }
    { mpz_class a {j}; Interval::negate(&a); assert(a == -10); }
    { long      a {k}; Interval::negate(&a); assert(a ==  -4); }

    { const mpz_class a {Interval::delta(Interval::Lohi { 3,  6})}; assert(a ==  3); }
    { const mpz_class a {Interval::delta(Interval::Lohi { 3,  7})}; assert(a ==  4); }
    { const mpz_class a {Interval::delta(Interval::Lohi { 4,  7})}; assert(a ==  3); }
    { const mpz_class a {Interval::delta(Interval::Lohi { 4,  8})}; assert(a ==  4); }
    { const mpz_class a {Interval::delta(Interval::Lohi {-6, -3})}; assert(a ==  3); }
    { const mpz_class a {Interval::delta(Interval::Lohi {-7, -3})}; assert(a ==  4); }
    { const mpz_class a {Interval::delta(Interval::Lohi {-7, -4})}; assert(a ==  3); }
    { const mpz_class a {Interval::delta(Interval::Lohi {-8, -4})}; assert(a ==  4); }
    { const mpz_class a {Interval::delta(Interval::Lohi {-3,  6})}; assert(a ==  9); }
    { const mpz_class a {Interval::delta(Interval::Lohi {-3,  7})}; assert(a == 10); }
    { const mpz_class a {Interval::delta(Interval::Lohi {-4,  7})}; assert(a == 11); }
    { const mpz_class a {Interval::delta(Interval::Lohi {-4,  8})}; assert(a == 12); }
    { const mpz_class a {Interval::delta(Interval::Lohi {-1,  5})}; assert(a ==  6); }
    { const mpz_class a {Interval::delta(Interval::Lohi {-1,  6})}; assert(a ==  7); }
    { const mpz_class a {Interval::delta(Interval::Lohi {-1,  7})}; assert(a ==  8); }
    { const mpz_class a {Interval::delta(Interval::Lohi {-1,  8})}; assert(a ==  9); }
    { const mpz_class a {Interval::delta(Interval::Lohi { 0,  5})}; assert(a ==  5); }
    { const mpz_class a {Interval::delta(Interval::Lohi { 0,  6})}; assert(a ==  6); }
    { const mpz_class a {Interval::delta(Interval::Lohi { 0,  7})}; assert(a ==  7); }
    { const mpz_class a {Interval::delta(Interval::Lohi { 0,  8})}; assert(a ==  8); }
    { const mpz_class a {Interval::delta(Interval::Lohi { 1,  5})}; assert(a ==  4); }
    { const mpz_class a {Interval::delta(Interval::Lohi { 1,  6})}; assert(a ==  5); }
    { const mpz_class a {Interval::delta(Interval::Lohi { 1,  7})}; assert(a ==  6); }
    { const mpz_class a {Interval::delta(Interval::Lohi { 1,  8})}; assert(a ==  7); }
    { const mpz_class a {Interval::delta(Interval::Lohi { 2,  5})}; assert(a ==  3); }
    { const mpz_class a {Interval::delta(Interval::Lohi { 2,  6})}; assert(a ==  4); }
    { const mpz_class a {Interval::delta(Interval::Lohi { 2,  7})}; assert(a ==  5); }
    { const mpz_class a {Interval::delta(Interval::Lohi { 2,  8})}; assert(a ==  6); }
    { const mpz_class a {Interval::delta(Interval::Lohi {-1, -1})}; assert(a ==  0); }
    { const mpz_class a {Interval::delta(Interval::Lohi {-1,  0})}; assert(a ==  1); }
    { const mpz_class a {Interval::delta(Interval::Lohi {-1,  1})}; assert(a ==  2); }
    { const mpz_class a {Interval::delta(Interval::Lohi { 0,  0})}; assert(a ==  0); }
    { const mpz_class a {Interval::delta(Interval::Lohi { 0,  1})}; assert(a ==  1); }
    { const mpz_class a {Interval::delta(Interval::Lohi { 1,  1})}; assert(a ==  0); }
    { const mpz_class a {Interval::mid(Interval::Lohi { 3,  6})}; assert(a ==  4); }
    { const mpz_class a {Interval::mid(Interval::Lohi { 3,  7})}; assert(a ==  5); }
    { const mpz_class a {Interval::mid(Interval::Lohi { 4,  7})}; assert(a ==  6); }
    { const mpz_class a {Interval::mid(Interval::Lohi { 4,  8})}; assert(a ==  6); }
    { const mpz_class a {Interval::mid(Interval::Lohi {-6, -3})}; assert(a == -4); }
    { const mpz_class a {Interval::mid(Interval::Lohi {-7, -3})}; assert(a == -5); }
    { const mpz_class a {Interval::mid(Interval::Lohi {-7, -4})}; assert(a == -6); }
    { const mpz_class a {Interval::mid(Interval::Lohi {-8, -4})}; assert(a == -6); }
    { const mpz_class a {Interval::mid(Interval::Lohi {-3,  6})}; assert(a ==  2); }
    { const mpz_class a {Interval::mid(Interval::Lohi {-3,  7})}; assert(a ==  2); }
    { const mpz_class a {Interval::mid(Interval::Lohi {-4,  7})}; assert(a ==  2); }
    { const mpz_class a {Interval::mid(Interval::Lohi {-4,  8})}; assert(a ==  2); }
    { const mpz_class a {Interval::mid(Interval::Lohi {-1,  5})}; assert(a ==  2); }
    { const mpz_class a {Interval::mid(Interval::Lohi {-1,  6})}; assert(a ==  2); }
    { const mpz_class a {Interval::mid(Interval::Lohi {-1,  7})}; assert(a ==  3); }
    { const mpz_class a {Interval::mid(Interval::Lohi {-1,  8})}; assert(a ==  4); }
    { const mpz_class a {Interval::mid(Interval::Lohi { 0,  5})}; assert(a ==  2); }
    { const mpz_class a {Interval::mid(Interval::Lohi { 0,  6})}; assert(a ==  3); }
    { const mpz_class a {Interval::mid(Interval::Lohi { 0,  7})}; assert(a ==  4); }
    { const mpz_class a {Interval::mid(Interval::Lohi { 0,  8})}; assert(a ==  4); }
    { const mpz_class a {Interval::mid(Interval::Lohi { 1,  5})}; assert(a ==  3); }
    { const mpz_class a {Interval::mid(Interval::Lohi { 1,  6})}; assert(a ==  4); }
    { const mpz_class a {Interval::mid(Interval::Lohi { 1,  7})}; assert(a ==  4); }
    { const mpz_class a {Interval::mid(Interval::Lohi { 1,  8})}; assert(a ==  4); }
    { const mpz_class a {Interval::mid(Interval::Lohi { 2,  5})}; assert(a ==  4); }
    { const mpz_class a {Interval::mid(Interval::Lohi { 2,  6})}; assert(a ==  4); }
    { const mpz_class a {Interval::mid(Interval::Lohi { 2,  7})}; assert(a ==  4); }
    { const mpz_class a {Interval::mid(Interval::Lohi { 2,  8})}; assert(a ==  5); }
    { const mpz_class a {Interval::mid(Interval::Lohi {-1, -1})}; assert(a == -1); }
    { const mpz_class a {Interval::mid(Interval::Lohi {-1,  0})}; assert(a ==  0); }
    { const mpz_class a {Interval::mid(Interval::Lohi {-1,  1})}; assert(a ==  0); }
    { const mpz_class a {Interval::mid(Interval::Lohi { 0,  0})}; assert(a ==  0); }
    { const mpz_class a {Interval::mid(Interval::Lohi { 0,  1})}; assert(a ==  0); }
    { const mpz_class a {Interval::mid(Interval::Lohi { 1,  1})}; assert(a ==  1); }
    { const mpz_class a {Interval::dev(Interval::Lohi { 3,  6})}; assert(a ==  2); }
    { const mpz_class a {Interval::dev(Interval::Lohi { 3,  7})}; assert(a ==  2); }
    { const mpz_class a {Interval::dev(Interval::Lohi { 4,  7})}; assert(a ==  2); }
    { const mpz_class a {Interval::dev(Interval::Lohi { 4,  8})}; assert(a ==  2); }
    { const mpz_class a {Interval::dev(Interval::Lohi {-6, -3})}; assert(a ==  2); }
    { const mpz_class a {Interval::dev(Interval::Lohi {-7, -3})}; assert(a ==  2); }
    { const mpz_class a {Interval::dev(Interval::Lohi {-7, -4})}; assert(a ==  2); }
    { const mpz_class a {Interval::dev(Interval::Lohi {-8, -4})}; assert(a ==  2); }
    { const mpz_class a {Interval::dev(Interval::Lohi {-3,  6})}; assert(a ==  5); }
    { const mpz_class a {Interval::dev(Interval::Lohi {-3,  7})}; assert(a ==  5); }
    { const mpz_class a {Interval::dev(Interval::Lohi {-4,  7})}; assert(a ==  6); }
    { const mpz_class a {Interval::dev(Interval::Lohi {-4,  8})}; assert(a ==  6); }
    { const mpz_class a {Interval::dev(Interval::Lohi {-1,  5})}; assert(a ==  3); }
    { const mpz_class a {Interval::dev(Interval::Lohi {-1,  6})}; assert(a ==  4); }
    { const mpz_class a {Interval::dev(Interval::Lohi {-1,  7})}; assert(a ==  4); }
    { const mpz_class a {Interval::dev(Interval::Lohi {-1,  8})}; assert(a ==  5); }
    { const mpz_class a {Interval::dev(Interval::Lohi { 0,  5})}; assert(a ==  3); }
    { const mpz_class a {Interval::dev(Interval::Lohi { 0,  6})}; assert(a ==  3); }
    { const mpz_class a {Interval::dev(Interval::Lohi { 0,  7})}; assert(a ==  4); }
    { const mpz_class a {Interval::dev(Interval::Lohi { 0,  8})}; assert(a ==  4); }
    { const mpz_class a {Interval::dev(Interval::Lohi { 1,  5})}; assert(a ==  2); }
    { const mpz_class a {Interval::dev(Interval::Lohi { 1,  6})}; assert(a ==  3); }
    { const mpz_class a {Interval::dev(Interval::Lohi { 1,  7})}; assert(a ==  3); }
    { const mpz_class a {Interval::dev(Interval::Lohi { 1,  8})}; assert(a ==  4); }
    { const mpz_class a {Interval::dev(Interval::Lohi { 2,  5})}; assert(a ==  2); }
    { const mpz_class a {Interval::dev(Interval::Lohi { 2,  6})}; assert(a ==  2); }
    { const mpz_class a {Interval::dev(Interval::Lohi { 2,  7})}; assert(a ==  3); }
    { const mpz_class a {Interval::dev(Interval::Lohi { 2,  8})}; assert(a ==  3); }
    { const mpz_class a {Interval::dev(Interval::Lohi {-1, -1})}; assert(a ==  0); }
    { const mpz_class a {Interval::dev(Interval::Lohi {-1,  0})}; assert(a ==  1); }
    { const mpz_class a {Interval::dev(Interval::Lohi {-1,  1})}; assert(a ==  1); }
    { const mpz_class a {Interval::dev(Interval::Lohi { 0,  0})}; assert(a ==  0); }
    { const mpz_class a {Interval::dev(Interval::Lohi { 0,  1})}; assert(a ==  1); }
    { const mpz_class a {Interval::dev(Interval::Lohi { 1,  1})}; assert(a ==  0); }

    return 0;

}

