#include "../../src/stringific.hh"
#include <string>
#include <cassert>
#include <gmpxx.h>

int main()
{
    {
        const mpz_class a {0x12000}; const mp_bitcnt_t prec {0x10}; const std::string s {Interval::stringify(prec, a)};
        assert(s == "              1.2000\n");
    }
    {
        const mpz_class a {-0x12000}; const mp_bitcnt_t prec {0x10}; const std::string s {Interval::stringify(prec, a)};
        assert(s == "             -1.2000\n");
    }
    {
        const mpz_class a {0}; const mp_bitcnt_t prec {0x10}; const std::string s {Interval::stringify(prec, a)};
        assert(s == "              0.0000\n");
    }
    {
        const mpz_class a {0x12000}; const mp_bitcnt_t prec {0xf}; const std::string s {Interval::stringify(prec, a)};
        assert(s == "              2.4000\n");
    }
    {
        const mpz_class a {0x12000}; const mp_bitcnt_t prec {0x12}; const std::string s {Interval::stringify(prec, a)};
        assert(s == "              0.4800 0\n");
    }
    {
        const mpz_class a {0x12000}; const mp_bitcnt_t prec {0x92}; const std::string s {Interval::stringify(prec, a)};
        assert(s == "              0.0000 0000 0000 0000 0000 0000 0000 0000\n                4800 0\n");
    }
    {
        const mpz_class a {-0x12000}; const mp_bitcnt_t prec {0x92}; const std::string s {Interval::stringify(prec, a)};
        assert(s == "             -0.0000 0000 0000 0000 0000 0000 0000 0000\n                4800 0\n");
    }
    {
        const mpz_class a {0x30}; const mp_bitcnt_t prec {0x4}; const std::string s {Interval::stringify(prec, a)};
        assert(s == "              3.0\n");
    }
    {
        const mpz_class a {0x3}; const mp_bitcnt_t prec {0}; const std::string s {Interval::stringify(prec, a)};
        assert(s == "              3.0\n");
    }
    {
        const mpz_class a {-0xdf376a1}; const mp_bitcnt_t prec {0x0}; const std::string s {Interval::stringify(prec, a)};
        assert(s == "      -df3 76a1.0\n");
    }
    {
        const mpz_class a {-0xdf376a1}; const mp_bitcnt_t prec {0x4}; const std::string s {Interval::stringify(prec, a)};
        assert(s == "       -df 376a.1\n");
    }
    {
        const mpz_class a {-0xdf376a1}; const mp_bitcnt_t prec {0x8}; const std::string s {Interval::stringify(prec, a)};
        assert(s == "        -d f376.a1\n");
    }
    {
        const mpz_class a {-0xdf376a1}; const mp_bitcnt_t prec {0xc}; const std::string s {Interval::stringify(prec, a)};
        assert(s == "          -df37.6a1\n");
    }
    {
        const mpz_class a {-0xdf376a1}; const mp_bitcnt_t prec {0x10}; const std::string s {Interval::stringify(prec, a)};
        assert(s == "           -df3.76a1\n");
    }
    {
        const mpz_class a {0x12000}; const mp_bitcnt_t prec {0x10}; const std::string s {Interval::stringify(prec, a, Interval::Stringify_style::integer)};
        assert(s == "         1 2000\n");
    }
    {
        const mpz_class a {-0x12000}; const mp_bitcnt_t prec {0x10}; const std::string s {Interval::stringify(prec, a, Interval::Stringify_style::integer)};
        assert(s == "        -1 2000\n");
    }
    {
        const mpz_class a {0x50}; const mp_bitcnt_t prec {0x10}; const std::string s {Interval::stringify(prec, a, Interval::Stringify_style::integer)};
        assert(s == "             50\n");
    }
    {
        const mpz_class a {-0x50}; const mp_bitcnt_t prec {0x10}; const std::string s {Interval::stringify(prec, a, Interval::Stringify_style::integer)};
        assert(s == "            -50\n");
    }
    {
        const mpz_class a {0x12000}; const mp_bitcnt_t prec {0}; const std::string s {Interval::stringify(prec, a, Interval::Stringify_style::integer)};
        assert(s == "         1 2000\n");
    }
    {
        const mpz_class a {-0x12000}; const mp_bitcnt_t prec {0}; const std::string s {Interval::stringify(prec, a, Interval::Stringify_style::integer)};
        assert(s == "        -1 2000\n");
    }
    {
        const mpz_class a {0x50}; const mp_bitcnt_t prec {0}; const std::string s {Interval::stringify(prec, a, Interval::Stringify_style::integer)};
        assert(s == "             50\n");
    }
    {
        const mpz_class a {-0x50}; const mp_bitcnt_t prec {0}; const std::string s {Interval::stringify(prec, a, Interval::Stringify_style::integer)};
        assert(s == "            -50\n");
    }
    {
        const mpz_class a {0x50}; const mp_bitcnt_t prec {1}; const std::string s {Interval::stringify(prec, a, Interval::Stringify_style::integer)};
        assert(s == "             50\n");
    }
    {
        const mpz_class a {0x50}; const mp_bitcnt_t prec {2}; const std::string s {Interval::stringify(prec, a, Interval::Stringify_style::integer)};
        assert(s == "             50\n");
    }
    {
        const mpz_class a {0x50}; const mp_bitcnt_t prec {3}; const std::string s {Interval::stringify(prec, a, Interval::Stringify_style::integer)};
        assert(s == "             50\n");
    }
    {
        const mpz_class a {0x50}; const mp_bitcnt_t prec {4}; const std::string s {Interval::stringify(prec, a, Interval::Stringify_style::integer)};
        assert(s == "             50\n");
    }
    {
        const mpz_class a {0x50}; const mp_bitcnt_t prec {5}; const std::string s {Interval::stringify(prec, a, Interval::Stringify_style::integer)};
        assert(s == "             50\n");
    }
    return 0;
}

